unit Prometheus.Client.Collectors.Counter;

interface

uses
  Prometheus.Client.SimpleCollector,
  Prometheus.Client.Labels,
  Prometheus.Client.Samples;

type

{ TCounterChild }

  TCounterChild = class
  private
    FValue: Double;
  public
    procedure Inc(const AAmount: Double = 1);
    property Value: Double read FValue;
  end;

{ TCounter }

  TCounter = class (TSimpleCollector<TCounterChild>)
  private
    function GetValue: Double;
  protected
    function CreateNewChild: TCounterChild; override;
  public
    procedure Inc(const AAmount: Double = 1);
    function Collect: TArray<TMetricSamples>; override;
    function GetNames: TArray<string>; override;
    property Value: Double read GetValue;
  end;

implementation

uses
  System.Classes,
  System.SysUtils,
  Prometheus.Client.Resources;

{ TCounterChild }

procedure TCounterChild.Inc(const AAmount: Double);
begin
  if AAmount <= 0 then
    raise EArgumentOutOfRangeException.Create(StrErrAmountLessThanZero);
  FValue := FValue + AAmount;
end;

{ TCounter }

function TCounter.Collect: TArray<TMetricSamples>;
begin
  SetLength(Result, 1);
  var LMetric := PMetricSamples(@Result[0]);
  LMetric^.MetricName := Self.Name;
  LMetric^.MetricHelp := Self.Help;
  LMetric^.MetricType := 'counter';
  SetLength(LMetric^.Samples, ChildrenCount);
  var LIndex := 0;
  EnumChildren(
    procedure (const ALabelValues: TLabelValues; AChild: TCounterChild)
    begin
      var LSample := PSample(@LMetric^.Samples[LIndex]);
      LSample^.MetricName := Self.Name;
      LSample^.LabelNames := Self.LabelNames;
      LSample^.LabelValues := ALabelValues;
      LSample^.TimeStamp := 0;
      LSample^.Value := AChild.Value;
      System.Inc(LIndex);
    end
  );
end;

function TCounter.CreateNewChild: TCounterChild;
begin
  Result := TCounterChild.Create();
end;

function TCounter.GetNames: TArray<string>;
begin
  Result := [Name];
end;

function TCounter.GetValue: Double;
begin
  Result := GetUnlabelledChild.Value;
end;

procedure TCounter.Inc(const AAmount: Double);
begin
  GetUnlabelledChild.Inc(AAmount);
end;

end.
