unit Prometheus.Client.Labels;

interface

uses
  System.Generics.Defaults;

type

  TLabelNames = TArray<string>;
  TLabelValues = TArray<string>;

  TLabelConsts = class sealed
  private
    class var FEmptyNames: TLabelNames;
  public
    class constructor Create;
    class property EmptyNames: TLabelNames read FEmptyNames;
  end;

{ TLabelValidator }

  TLabelValidator = class sealed
  private const
    NamePattern: string = '^[a-zA-Z_][a-zA-Z0-9_]*$';
    ReservedPattern = '^__.*$';
  public
    class procedure CheckLabel(const AName: string);
    class procedure CheckLabels(const ANames: TLabelNames);
  end;

{ TLabelNamesEqualityComparer }

  TLabelNamesEqualityComparer = class (TEqualityComparer<TArray<string>>)
   public
    function Equals(const Left, Right: TArray<string>): Boolean; override;
    function GetHashCode(const Value: TArray<string>): Integer; override;
   end;

implementation

uses
  System.Hash,
  System.RegularExpressions,
  System.StrUtils,
  System.SysUtils,
  Prometheus.Client.Resources;

{ TLabelValidator }

class procedure TLabelValidator.CheckLabel(const AName: string);
begin
  if Length(AName) <= 0 then
    raise EArgumentException.Create(StrErrEmptyLabelName);
  if not TRegEx.IsMatch(AName, NamePattern) then
    raise EArgumentException.Create(StrErrInvalidLabelName);
  if TRegEx.IsMatch(AName, ReservedPattern) then
    raise EArgumentException.Create(StrErrReservedLabelName);
end;

class procedure TLabelValidator.CheckLabels(const ANames: TLabelNames);
begin
  for var LName in ANames do
    CheckLabel(LName);
end;

{ TLabelConsts }

class constructor TLabelConsts.Create;
begin
  SetLength(FEmptyNames, 0);
end;

{ TLabelNamesEqualityComparer }

function TLabelNamesEqualityComparer.Equals(const Left, Right: TArray<string>): Boolean;
begin
  if Length(Left) <> Length(Right) then
  begin
    Result := False;
    Exit;
  end;
  for var LIndex := 0 to Pred(Length(Left)) do
    if not SameText(Left[LIndex], Right[LIndex]) then
    begin
      Result := False;
      Exit;
    end;
  Result := True;
end;

function TLabelNamesEqualityComparer.GetHashCode(const Value: TArray<string>): Integer;
begin
  var LText := '';
  for var LIndex := 0 to Pred(Length(Value)) do
    LText := LText + AnsiLowerCase(Value[LIndex]);
  Result := THashFNV1a32.GetHashValue(PChar(LText)^, SizeOf(Char) * Length(LText));
end;

end.
